/*
 * stepper.c
 *    This program uses the socket interface to pigpiod to move a stepper motor
 *    Copyright U. Raich May. 2017
 *    This program is part of a course on embedded systems held
 *    at the University of Cape Coast, Ghana in 2017
 *    It is released under the GNU Public License 
 *    For details please see https://www.gnu.org/licenses/gpl.html
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <stdbool.h>
#include <pigpiod_if2.h>

#define PHASE1 27
#define PHASE2 22
#define PHASE3 18
#define PHASE4 17

#define OFF     0
#define ON      1

#define NO_OF_STEPS 4096

/*
bool fullStepSequence[4][4] = {{1,1,1,0},
			       {0,0,1,1},
			       {1,0,0,1},
			       {0,1,0,0}};
*/

bool halfStepSequence[8][4] = {{1,0,0,0},
			       {1,1,0,0},
			       {0,1,0,0},
			       {0,1,1,0},
			       {0,0,1,0},
			       {0,0,1,1},
			       {0,0,0,1},			       
			       {1,0,0,1}};
int phases[4] = {PHASE1, PHASE2, PHASE3, PHASE4};

void oneCycle(int fd) {
  int i,j;
  for (i=0;i<8;i++) {
    for (j=0;j<4;j++) 
      gpio_write(fd, phases[j],halfStepSequence[i][j]);
    //      printf(" i: %d, j: %d , phase: %d, value: %d\n",
    //	     i,j,phases[j], fullStepSequence[i][j]);
    time_sleep(0.0020);
  }
}

int main(int argc, char ** argv) {
  int i,errCode;
  int fd=-1;
  
  if ((fd=pigpio_start(NULL,NULL)) < 0) {
    printf("halfStepForward: Could not open connection to pigpiod\n");
    printf("%s\n",pigpio_error(fd));
    return -1;
  }
  
  printf("halfStepForward: Connection to pigpiod successfully opened\n");

  for (i=0;i<4;i++) {
    if ((errCode=set_mode(fd,phases[i],PI_OUTPUT)) < 0) {
      printf("stepper: Could not set phase %d to output\n",i);
      printf("%s\n",pigpio_error(errCode));
      return -1;
    }
  }
  /*
  gpio_write(fd,PHASE1,0);
  gpio_write(fd,PHASE2,0);
  gpio_write(fd,PHASE3,0);
  gpio_write(fd,PHASE4,1);
  */
  for (i=0;i<NO_OF_STEPS/8;i++) 
    oneCycle(fd);  
  return 0;
}
